/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 19/11/22.
//

#ifndef FGGL_ASSETS_PACKED_DIRECT_HPP
#define FGGL_ASSETS_PACKED_DIRECT_HPP

#include <functional>
#include <map>

#include "fggl/assets/types.hpp"
#include "fggl/util/safety.hpp"
#include "fggl/util/guid.hpp"
#include "fggl/modules/module.hpp"

/**
 * Raw Checkin.
 *
 * This is a version of the checkin system where the check-in functions are shown a raw block of memory and its their
 * job to parse and load something meaningful from that.
 */
namespace fggl::assets {

	class RawCheckin {
		public:
			constexpr const static auto service = modules::make_service("fggl::assets::checkin");
			using DecodeAndCheckFunc = std::function<void(AssetGUID, MemoryBlock& block)>;

			void check(AssetID, AssetTypeID, MemoryBlock& block) const;

			inline void check(int64_t id, uint64_t type, MemoryBlock& block ) const {
				check( AssetID::make(id), AssetTypeID::make(type), block );
			}

			inline void setCheckin(AssetTypeID type, DecodeAndCheckFunc func) {
				m_check[type] = func;
			}

		private:
			std::map<AssetTypeID, DecodeAndCheckFunc> m_check;
	};

}

#endif //FGGL_ASSETS_PACKED_DIRECT_HPP
