/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 27/06/22.
//

#ifndef FGGL_AUDIO_OPENAL_MODULE_HPP
#define FGGL_AUDIO_OPENAL_MODULE_HPP

#include <array>
#include <string>

#include "fggl/assets/module.hpp"
#include "fggl/assets/packed/module.hpp"

#include "fggl/audio/audio.hpp"
#include "fggl/audio/openal/audio.hpp"

namespace fggl::audio {

	constexpr auto OGG_VORBIS = assets::AssetType::make("audio/vorbis");
	constexpr auto RES_OGG_VORBIS = assets::from_mime("audio/vorbis");

    //! an audio module which uses openal(-soft) as a backend.
	struct OpenAL {
		constexpr static const char *name = "fggl::audio::OpenAL";
		constexpr static const std::array<modules::ServiceName, 1> provides = {
			SERVICE_AUDIO_PLAYBACK
		};
		constexpr static const std::array<modules::ServiceName, 2> depends = {
			assets::AssetManager::service,
			assets::CheckinAdapted::service
		};
		static bool factory(modules::ServiceName name, modules::Services &serviceManager);
	};


} // namespace fggl::audio

#endif //FGGL_AUDIO_OPENAL_MODULE_HPP
