/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 18/10/22.
//

#ifndef FGGL_DATA_ASSIMP_MODULE_HPP
#define FGGL_DATA_ASSIMP_MODULE_HPP

#include "fggl/modules/module.hpp"
#include "fggl/assets/loader.hpp"
#include "fggl/assets/packed/module.hpp"
#include "fggl/data/texture.hpp"

namespace fggl::data::models {

	constexpr auto MODEL_PROVIDER = modules::make_service("fggl::data::Model");
	constexpr auto ASSIMP_MODEL = assets::AssetType::make("model::assimp");

	constexpr auto MIME_JPG = assets::from_mime("image/jpeg");
	constexpr auto MIME_PNG = assets::from_mime("image/png");
	constexpr auto MIME_OBJ = assets::from_mime("model/obj");
	constexpr auto MIME_FBX = assets::from_mime("model/fbx");

	constexpr auto MODEL_MULTI3D = assets::make_asset_type("model/multi3D");
	constexpr auto TEXTURE_RGBA = assets::make_asset_type("texture/rgba");

	// old-style loaders
	assets::AssetRefRaw load_assimp_model(assets::Loader* loader, const assets::AssetID& guid, const assets::LoaderContext& data, void* userPtr);
	assets::AssetRefRaw load_assimp_texture(assets::Loader* loader, const assets::AssetID& guid, const assets::LoaderContext& data, void* userPtr);

	// new style loaders (textures)
	bool load_tex_stb(const std::filesystem::path& filePath, assets::MemoryBlock& block);
	assets::AssetTypeID is_tex_stb(const std::filesystem::path& filePath);

	// new style loaders (models)
	assets::AssetTypeID is_model_assimp(const std::filesystem::path& filePath);
	bool extract_requirements(const std::string& packName, const std::filesystem::path& packRoot, assets::ResourceRecord& rr);

	struct AssimpModule {
		constexpr static const char *name = "fggl::data::Assimp";
		constexpr static const std::array<modules::ServiceName, 1> provides = {
			MODEL_PROVIDER
		};
		constexpr static const std::array<modules::ServiceName, 2> depends = {
			assets::Loader::service,
			assets::CheckinAdapted::service
		};
		static bool factory(modules::ServiceName service, modules::Services &serviceManager);
	};

}

namespace fggl::data {
	using AssimpLoader = models::AssimpModule;
}

#endif //FGGL_DATA_ASSIMP_MODULE_HPP
