/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 27/06/22.
//

#ifndef FGGL_MODULES_MODULE_HPP
#define FGGL_MODULES_MODULE_HPP

#include <string>
#include <vector>
#include <functional>
#include <map>
#include <memory>

#include "fggl/util/safety.hpp"
#include "service.hpp"

namespace fggl::modules {

	template<typename T>
	concept ModuleType = requires(T type) {
		{ T::provides };
		{ T::depends };
	};

	using ModuleIdentifier = std::string;

	using ServiceFactory = std::function<bool(ServiceName , Services &)>;
	struct Config {
		ModuleIdentifier name;
		std::vector<ServiceName> provides;
		std::vector<ServiceName> depends;
		ServiceFactory factory = nullptr;
	};

	class Module {
		public:
			virtual ~Module() = default;

			// copying modules is bad
			Module(const Module&) = delete;
			Module& operator=(const Module&) = delete;

			// moving modules is bad
			Module(Module&&) = delete;
			Module& operator=(Module&&) = delete;

			virtual auto create(ServiceName, Services&) -> bool = 0;
	};

} // namespace fggl::modules

#endif //FGGL_MODULES_MODULE_HPP
